/*
This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "EnergyModelC.h"

#include <imagelib/ImageI.h>
#include <meshlib/MexelC.h>
#include <meshlib/MeshC.h>

#include <blitz/tinyvec-et.h>

using namespace dsmlib;
using namespace meshlib;
using namespace blitz;

/*EnergyModelC::EnergyModelC( MLFloat lambda ) throw ()
: m_lambda( lambda )
{
}

EnergyModelC::~EnergyModelC() throw ()
{
}*/

meshlib::MLFloat EnergyModelC::getMexelEnergy(
	const meshlib::MexelC & rMexel,
	const meshlib::MeshC & rMesh,
	const imagelib::ImageI & rImage,
	meshlib::MLFloat lambda ) throw ()
{
	return
		lambda * getInternal( rMexel, rMesh, rImage ) +
		(1.0 - lambda) * getExternal( rMexel, rMesh, rImage );
}

meshlib::MLFloat EnergyModelC::getInternal(
	const meshlib::MexelC & rMexel,
	const meshlib::MeshC & rMesh,
	const imagelib::ImageI & rImage ) throw ()
{
	MLVector distance( 0.0, 0.0, 0.0 );
	
	for ( int j = 0; j < 3; ++j )
		distance +=
			(rMexel.getAlpha()(j)) * (rMexel.getNeighbor(j)->getLocation());

	distance -= rMexel.getLocation();
		 
	return ( sum( distance * distance ) ) / rMesh.getAverageArea();
	
}

meshlib::MLFloat EnergyModelC::getExternal(
	const meshlib::MexelC & rMexel,
	const meshlib::MeshC & rMesh,
	const imagelib::ImageI & rImage ) throw ()
{
	TinyVector<MLFloat,3> locImageCoords =
		( rMexel.getLocation() + rMesh.getReferencePoint() );
		
	const TinyVector<int,3> & dim( rImage.getDimensions() );
			
	TinyVector<int,3> locRounded(
		(int)( locImageCoords(0) +0.5 ),
		(int)( locImageCoords(1) +0.5 ),
		(int)( locImageCoords(2) +0.5 ));
		
	for ( int i = 0; i < 3; ++i )
	{
/*		if ( locRounded(i) < 0 )
			locRounded(i) = 0;
		else if ( locRounded(i) >= dim(i) )
			locRounded(i) = dim(i) - 1;*/
		if ( locRounded(i) < 0 || locRounded(i) >= dim(i) )
//	  if ( locRounded(i) < 0 || locRounded(i) > (dim(i) -1) )
	    return 1.0; 
	}

	return 1.0 - ( ( *(rImage.getImageData()) )( locRounded ) );	
}

