/*
This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef DSMLIB_ENERGYMODELC_H
#define DSMLIB_ENERGYMODELC_H

#include <meshlib/typedefs.h>

namespace meshlib
{
	class MexelC;
	class MeshC;
};

namespace imagelib
{
	class ImageI;
};

namespace dsmlib
{
	/**Encapsulates the energy calculations of the DSM algorithm.
	 */
	class EnergyModelC
	{
	public:	

		/**Returns the total energy of a mexel.
		 */
		static meshlib::MLFloat getMexelEnergy(
			const meshlib::MexelC & rMexel,
			const meshlib::MeshC & rMesh,
			const imagelib::ImageI & rImage,
			meshlib::MLFloat lambda ) throw ();
		
	private:

		/**Returns the internal energy of a mexel.
		 */
		static meshlib::MLFloat getInternal( 
			const meshlib::MexelC & rMexel,
			const meshlib::MeshC & rMesh,
			const imagelib::ImageI & rImage ) throw ();

		/**Returns the external energy of a mexel.
		 */
		static meshlib::MLFloat getExternal(
			const meshlib::MexelC & rMexel,
			const meshlib::MeshC & rMesh,
			const imagelib::ImageI & rImage ) throw ();
	};
};

#endif //DSMLIB_ENERGYMODELC_H
