/*
This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "SearchSpaceC.h"

#include <meshlib/MexelC.h>
//#include <meshlib/

#include <blitz/tinyvec-et.h>

using namespace dsmlib;
using namespace meshlib;

SearchSpaceC::SearchSpaceC(
/*		 	const meshlib::MexelC & rMexel,
			int searchDirection,
			int J,
			int K,
			meshlib::MLFloat L,
			meshlib::MLFloat D */) throw ()
{
	m_size = 0;
	m_pLocationArray = NULL;
//	recalculate(rMexel,searchDirection,J,K,L,D);
}

void SearchSpaceC::recalculate(
 	const meshlib::MexelC & rMexel,
	int searchDirection,
	int J,
	int K,
	meshlib::MLFloat L,
	meshlib::MLFloat D ) throw ()
{
	unsigned int newSize = 3*J*K + J + 1;
	if ( newSize != m_size )
	{
		m_size = newSize;
		if ( m_pLocationArray )
			delete [] m_pLocationArray;
		m_pLocationArray = new MLVector[ m_size ];
		m_locations.clear();
		for ( unsigned int i = 0; i < m_size; ++i )
			m_locations.push_back( & ( m_pLocationArray[ i ] ) );
	}

	MLVector mexLoc( rMexel.getLocation() );
	MLFloat mexLen = sqrt( sum( mexLoc * mexLoc ) );   
	// MLFloat mexLen=  sqrt(mexLoc[0]*mexLoc[0] + mexLoc[1]*mexLoc[1] +  mexLoc[2]*mexLoc[2]);
	MLVector n1Loc( rMexel.getNeighbor(0)->getLocation() );
	MLVector n2Loc( rMexel.getNeighbor(1)->getLocation() );
	MLVector n3Loc( rMexel.getNeighbor(2)->getLocation() );
	MLVector wSum1( D * (n1Loc + n2Loc) );
	MLVector wSum2( D * (n1Loc + n3Loc) );
	MLVector wSum3( D * (n2Loc + n3Loc) );
	
	MLVector * pLocation = & ( m_pLocationArray[0] );
	
	*(pLocation++) = mexLoc;	
	
	MLFloat diff = (L * searchDirection) / mexLen;
	MLFloat scalar = 1.0 + diff;
			
		
	for ( int j = 1; j <= J; ++j )
	{
		*(pLocation++) = scalar * mexLoc;
		
		for( int k = 1; k <= K; ++k )
		{
			MLVector term1 = (1.0 - 2.0 * ((MLFloat)k) * D) * mexLoc;

			*(pLocation++) = (  scalar*(
				term1 + ( ((MLFloat)k) * wSum1 ) ));

			*(pLocation++) = (  scalar*(
				term1 + ( ((MLFloat)k) * wSum2 ) ));

			*(pLocation++) = (  scalar*(
				term1 + ( ((MLFloat)k) * wSum3 ) ));
		}
		scalar += diff;
	}
}

	
SearchSpaceC::~SearchSpaceC() throw ()
{
	if ( m_pLocationArray )
		delete [] m_pLocationArray;
}
	
/*unsigned int SearchSpaceC::getSize() const throw ()
{
	return m_locations.size();
}

meshlib::VectorConstIterator SearchSpaceC::begin() const throw ()
{
	return m_locations.begin();
}

meshlib::VectorConstIterator SearchSpaceC::end() const throw ()
{
	return m_locations.end();
}

meshlib::VectorIterator SearchSpaceC::begin() throw ()
{
	return m_locations.begin();
}

meshlib::VectorIterator SearchSpaceC::end() throw ()
{
	return m_locations.end();
}*/
		

