/*
This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef DSMLIB_SEARCHSPACEC_H
#define DSMLIB_SEARCHSPACEC_H

#include <meshlib/typedefs.h>
#include <vector>

/*namespace meshlib
{
	class MexelC;
};*/

namespace dsmlib
{
	/**Encapsulates the search space calculations of the DSM algorithm.
	 */
	class SearchSpaceC
	{
	public:	
	
		/**Construct an empty search space.
		 */
		SearchSpaceC() throw ();

		/**Construct a search space for a mexel.
		 */
/*		SearchSpaceC(
		 	const meshlib::MexelC & rMexel,
			int searchDirection,
			int J,
			int K,
			meshlib::MLFloat L,
			meshlib::MLFloat D ) throw ();*/
			
		/**A virtual destructor.
		 */
		virtual ~SearchSpaceC() throw ();
		
		/**Recalculates the search space for a new mexel.
		 */
		void recalculate(
		 	const meshlib::MexelC & rMexel,
			int searchDirection,
			int J,
			int K,
			meshlib::MLFloat L,
			meshlib::MLFloat D ) throw ();
					
		/**Returns the number of locations in the search space.
		 */
		unsigned int getSize() const throw ()
		{
			return m_size;
		};
		
		/**Returns an iterator to the first location in the search space.
		 */
		meshlib::VectorConstIterator begin() const throw ()
		{
			return m_locations.begin();
		};
		
		/**Returns an iterator to the one beyond last location in the
		 * search space.
		 */
		meshlib::VectorConstIterator end() const throw ()
		{
			return m_locations.end();
		};
		
		/**Returns an iterator to the first location in the search space.
		 */
		meshlib::VectorIterator begin() throw ()
		{
			return m_locations.begin();
		};
		
		/**Returns an iterator to the one beyond last location in the
		 * search space.
		 */
		meshlib::VectorIterator end() throw ()
		{
			return m_locations.end();
		};
		
	private:
	
		///Holds the locations in the search space.
		std::vector< meshlib::MLVector * > m_locations;
		meshlib::MLVector * m_pLocationArray;
		unsigned int m_size;
		
	};//class SearchSpaceC
};// namespace dsmlib

#endif //DSMLIB_SEARCHSPACEC_H

