/*
This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef CALLBACKHANDLERC_H
#define CALLBACKHANDLERC_H

#include <dsmlib/DSMCallbackI.h>
#include <dsmlib/DSMC.h>

#include <string>

/**Intermediate result callback handler.
 * This is an implementation of the DSMCallbackI interface that will save
 * intermediate result meshes and produce console output.
 */
class CallbackHandlerC : public dsmlib::DSMCallbackI
{
public:

	/**A helper method.
	 * Save the mesh to disk.
	 * @param rResults A reference to minimization results
	 * @param rFilename File to save to.
	 * @todo this should really be in UtilitiesC or somewhere...
	 */
	static void save_mesh(
		const dsmlib::DSMC::ResultsC & rResults,
		const std::string & rFilename );

	static void save_meshes(
		const dsmlib::DSMC::ResultsC & rResults,
		const std::string & rOuterFilename,
		const std::string & rInnerFilename );
		
	bool savedThisRound() throw ()
	{
		return m_savedThisRound;
	};
	
	/**Constructor.
	 * @param interval Store results every @a interval rounds
	 * (-1 for no save)
	 * @param rPattern Intermediate file name pattern
	 * @param rExt File name extension to use
	 * @param rDir Intermediate file output directory
	 * @param verbose Verbose output
	 * @param silent No output
	 */
	CallbackHandlerC(
		int interval,
		const std::string & rPattern,
		const std::string & rExt,
		const std::string & rDir,
		bool verbose,
		bool silent ) throw ();
		
	/**A non-virtual destructor.
	 */
	~CallbackHandlerC() throw ();
	
	/**The handler method.
	 * @param rResults A ref to minimization results
	 */
	void handle( const dsmlib::DSMC::ResultsC & rResults );
	
	void saveIntermediate( const dsmlib::DSMC::ResultsC & rResults );
	
private:

	///Save interval.
	int m_interval;
	
	///File name pattern.
	std::string m_pattern;

	///File name extension.
	std::string m_ext;
	
	///Output directory.
	std::string m_dir;
	
	///Verbose flag.
	bool m_verbose;
	
	///Silent flag.
	bool m_silent;
	
	///Used for round tracking.
	int m_round;

	///Counts saved files.
	int m_fileCounter;
	
	///Saved intermediate results this round?
	bool m_savedThisRound;
	
}; // class CallbackHandlerC

#endif //CALLBACKHANDLERC_H
