/*
This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef DUALSURFACEMIN_PARAMDEFSC_H
#define DUALSURFACEMIN_PARAMDEFSC_H

#include <paramlib/ParametersC.h>

/**Parameter definitions.
 * This class holds the static data that is used to initialize
 * paramlib::ParametersC with parameters specific to @a dualsurfacemin.
 */
class ParamDefsC
{

private:
		/**Parameter definitions as an array.
		 */
		static paramlib::ParametersC::DefinitionS definitions[];
		
		/**Holds the global parameter set.
		 */
		static paramlib::ParametersC * m_pParams;

public:
		/**Converts parameter definitions to a vector compatible with
		 * paramlib::ParametersC
		 */
		static std::vector<paramlib::ParametersC::DefinitionS> getDefs();
		
		/**Initialize the parameter set.
		 */
		static void initParams( int argc, char * argv[] );
		
		/**Returns the global parameter set.
		 */
		static paramlib::ParametersC & getParams();
		
		
		/**Parameter key identifiers.
		 */
		enum KeyIdE
		{
			OUTER_MESH = 1,
			OUTER_SCALE,
			OUTER_REF,
//			FIND_OUTER_REF,
			INNER_MESH,
			INNER_SCALE,
			INNER_REF,
//			FIND_INNER_REF,
			IMAGE,
			IMAGE_DIMENSIONS,
			IMAGE_DATA,
			IMAGE_SWAP_BYTES,
			IMAGE_NORMALIZE,

			OUTPUT_MESH,
			OUTPUT_OFF,
			OUTPUT_TRIANGULATED,
			OUTPUT_IMAGE_COORDS,
			IMED_INTERVAL,
			IMED_PATTERN,
			IMED_DIR,

			MAX_ROUNDS,
			TOLERANCE,
			VOLUME_RATIO,
			LAMBDA,
			GAMMA,
			SEARCH_J,
			SEARCH_K,
			SEARCH_L,
			SEARCH_D,

			HELP,
			VERBOSE,
			SILENT,

			CONFIG
			
		}; // enum KeyIdE
		
}; // class ParamDefsC

#endif //DUALSURFACEMIN_PARAMDEFSC_H
