/*
This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef IMAGELIB_IMAGEC_H
#define IMAGELIB_IMAGEC_H

#include "ImageI.h"

namespace imagelib
{
	/**A default implementation of the ImageI interface.
	 */
	class ImageC : virtual public ImageI
	{
	public:
		/**Constructs an ImageC object from a Blitz++ Array.
		 * Memory ownership is transferred.
		 */
		ImageC( blitz::Array<float,3> * pData ) throw() {
			m_pImageData = pData;
		}

		virtual ~ImageC() throw() {
			if( m_pImageData )
				delete m_pImageData;
		}

		const blitz::TinyVector<int,3> getDimensions() const throw() {
			blitz::TinyVector<int,3> dim( m_pImageData->ubound() );
			++dim(0);
			++dim(1);
			++dim(2);
			return dim;
		};

		void normalize() throw ();
		
		blitz::Array<float,3> * getImageData() throw() {
			return m_pImageData;
		};

		const blitz::Array<float,3> * getImageData() const throw() {
			return m_pImageData;
		};
//		virtual void setDimensions(const blitz::TinyVector<int,3> & rDimensions) throw();
//		virtual void setImageData( std::istream & rStream );

	private:
//		blitz::TinyVector<int,3> m_dimensions;
		blitz::Array<float,3> * m_pImageData;		
	};
};

#endif //IMAGELIB_IMAGEC_H
