/*
This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef IMAGELIB_RAWIMAGELOADERI_H
#define IMAGELIB_RAWIMAGELOADERI_H

#include "ImageLoaderI.h"
#include <iostream>

namespace imagelib
{
	/**Raw voxel data loader.
	 */
	class RAWImageLoaderC : virtual public ImageLoaderI
	{
	public:
		static RAWImageLoaderC & getInstance();
		virtual ~RAWImageLoaderC() throw();

		virtual const std::string & getDescription() const throw();		

		virtual bool accepts( const std::string & rFileName ) const throw();

		virtual bool accepts( const std::string & rFileName, const ImageI::ParametersC & rParams ) const throw();

		virtual ImageI * load( const std::string & rFileName, const ImageI::ParametersC & rParams ) const
			throw( iolib::ExceptionC );
			
		virtual ImageI * load( const std::string & rFileName ) const
			throw( iolib::ExceptionC );
			
		virtual void save( const ImageI * pProduct, const std::string & rFileName, const ImageI::ParametersC & rParams ) const
			throw( iolib::ExceptionC );
			
		virtual void save( const ImageI * pProduct, const std::string & rFileName ) const
			throw( iolib::ExceptionC );
			
	protected:
		RAWImageLoaderC() throw();

		/**A template function to automate loading of different data types.
		 * Loads @a count elements of type @a T from a stream.
		 * @param T A type to hold the loaded element
		 * @param N The size of the element to be loaded (in bytes)
		 * @param pData A buffer to load to
		 * @param rStream A stream to load from
		 * @param count The number of elements to load
		 * @param bSwap Swap byte order if true.
		 */
		template<typename T, int N>
		static void read_data( float *pData, std::istream & rStream,
			unsigned int count, bool bSwap );
	};
};

#endif //IMAGELIB_RAWIMAGELOADERI_H
