/*
This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef IOLIB_EXCEPTIONC_H
#define IOLIB_EXCEPTIONC_H

#include <exception>
#include <string>

namespace iolib
{
	/**An exception base class.
	 */
	class ExceptionC : public std::exception
	{
	public:
	
		/**Default constructor.
		 */
	    ExceptionC() throw() {};

		/**Constructor from a string.
		 */
	    ExceptionC( const std::string & rWhat ) throw()
	    	:  m_what( rWhat ) {};
	    
	    /**Copy constructor.
	     */
		ExceptionC( const ExceptionC & rException ) throw()
			: m_what( rException.m_what ) {};
		
		/**Assignment.
		 */	
	    ExceptionC& operator=( const ExceptionC & rException ) throw()
	    {
	    	if ( &rException != this )
	    		m_what = rException.m_what;
	    	return *this;
	    }

		/**A virtual destructor.
		 */
	    virtual ~ExceptionC() throw() {};

		/**Return the exception message.
		 */
	    virtual const char* what() const throw()
	    {
	    	return m_what.c_str();
	    };
	    
  private:
  		
  		/**Holds the exception message.
  		 */
  		std::string m_what;
  		
	}; // class ExceptionC
	
}; // namespace iolib

#endif // IOLIB_EXCEPTIONC_H
