/*
This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef MESHLIB_TYPEDEFS_H
#define MESHLIB_TYPEDEFS_H

#include <blitz/tinyvec.h>
#include <vector>

namespace meshlib
{
	/** meshlib floating point type.
	 * Change this to long double if you need more precision.
	 */
	typedef double MLFloat;
	
	/** meshlib vector type.
	 */
	typedef blitz::TinyVector<MLFloat,3> MLVector;
		
	// Forward declarations for iterator types.
	class MexelC;
	class FaceC;
	
	/** Iterator type
	 */
	typedef std::vector<MLVector*>::iterator VectorIterator;

	/** Iterator type
	 */
	typedef std::vector<MLVector*>::const_iterator VectorConstIterator;

	/** Iterator type
	 */
	typedef std::vector<MexelC*>::iterator MexelIterator;

	/** Iterator type
	 */
	typedef std::vector<MexelC*>::const_iterator MexelConstIterator;

	/** Iterator type
	 */
	typedef std::vector<FaceC*>::iterator FaceIterator;

	/** Iterator type
	 */
	typedef std::vector<FaceC*>::const_iterator FaceConstIterator;
	
}; // namespace meshlib

#endif //MESHLIB_TYPEDEFS_H
